//	Copyright 2007 Fred Ross
//
//	This file is part of MultiRoi.
//
//    MultiRoi is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    MultiRoi is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import ij.ImagePlus;
import ij.gui.*;
import java.util.*;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import com.vividsolutions.jts.geom.*;
import java.lang.reflect.Array;

/**
 * 
 */

/**
 * @author fross
 *
 */
public class MultiRoiStackCanvas extends ImageCanvas {
	public Map<Integer,List<Roi>> rois;
	protected ImagePlus imp;
	static private GeometryFactory gfact = new GeometryFactory();
	/**
	 * @param imp
	 */
	public MultiRoiStackCanvas(ImagePlus imp) {
		super(imp);
		this.imp = imp;
		this.rois = new HashMap<Integer,List<Roi>>(imp.getStackSize());
		for (int i = 1; i <= imp.getStackSize(); i++) 
			rois.put(i,new ArrayList<Roi>());
	}
	
	public void drawUnselectedRoi(Roi r, Graphics g) {
		g.setColor(Color.red);
		Rectangle rect = getSrcRect();
		double mag = getMagnification();
		int[] x = r.getPolygon().xpoints;
		int[] y = r.getPolygon().ypoints;
		for (int i = 1; i < r.getPolygon().npoints; i++) {
			int px = (int)Math.round((x[i-1]-rect.x)*mag);
			int py = (int)Math.round((y[i-1]-rect.y)*mag);
			int qx = (int)Math.round((x[i]-rect.x)*mag);
			int qy = (int)Math.round((y[i]-rect.y)*mag);
			g.drawLine(px, py, qx, qy);
		}
		int n = r.getPolygon().npoints;
		int px = (int)Math.round((x[n-1]-rect.x)*mag);
		int py = (int)Math.round((y[n-1]-rect.y)*mag);
		int qx = (int)Math.round((x[0]-rect.x)*mag);
		int qy = (int)Math.round((y[0]-rect.y)*mag);
		g.drawLine(px, py, qx, qy);
	}
	
	public void paint(Graphics g) {
		super.paint(g);
		final List<Roi> localRois = rois.get(imp.getCurrentSlice());
		for (Roi r : localRois) drawUnselectedRoi(r,g);
		if (imp.getRoi() != null) imp.getRoi().draw(g);
	}
	
	public void mousePressed(MouseEvent e) {
		Integer x = offScreenX(e.getX());
		Integer y = offScreenY(e.getY());
		Rectangle rect = getSrcRect();
		if (x < rect.x || x > rect.x + rect.width || y < rect.y || y > rect.y+rect.width) {
			super.mousePressed(e);
		} else {
			if (e.getButton() == MouseEvent.BUTTON3) {
				List<Roi> currentSliceRois = rois.get(imp.getCurrentSlice());
				Roi currentRoi = imp.getRoi();
				Roi newRoi = null;
				int j = 0;
				for (int i = 0; i < currentSliceRois.size(); i++) {
					try {
						Roi r = currentSliceRois.get(i);
						if (r == null) continue;
						if (roiToGeometry(r).contains(gfact.createPoint(new Coordinate(x,y)))) {
							newRoi = r;
							j = i;
							break;
						}
					} catch (Exception ex) {
						continue;
					}
				}
				
				if (acceptableRoi(newRoi)) {
					if (acceptableRoi(currentRoi)) {
						currentSliceRois.remove(j);
						currentSliceRois.add(currentRoi);
						imp.setRoi(newRoi);
					} else {
						currentSliceRois.remove(j);
						imp.setRoi(newRoi);
					}
				} else {
					if (acceptableRoi(currentRoi)) {
						currentSliceRois.add(currentRoi);
						imp.setRoi((Roi)null);
					} else {
						imp.setRoi((Roi)null);
					}
				}
			} else {
				Roi current = imp.getRoi();
				super.mousePressed(e);
				if (current != null) imp.setRoi(current);
			}
			imp.updateAndRepaintWindow();
		}
	}
	
	public boolean acceptableRoi(Roi r) {
		return(r != null && r.isArea() && r.getPolygon().npoints > 3 && roiToGeometry(r).getArea() > 0); 
	}
	
	public static Geometry roiToGeometry(Roi r) {
		java.awt.Polygon p = r.getPolygon();
		Coordinate[] cs = (Coordinate[]) Array.newInstance(Coordinate.class, p.npoints+1);
	    for (int i = 0; i < p.npoints; i++) {
	    	cs[i] = new Coordinate(p.xpoints[i], p.ypoints[i]);
	    }
	    cs[p.npoints] = new Coordinate(p.xpoints[0], p.ypoints[0]);
	    Geometry g = gfact.createPolygon(gfact.createLinearRing(cs), null);
	    return g;
	}
	
	public static Roi geometryToRoi(Geometry g) {
		Coordinate[] cs = g.getCoordinates();
		int n = cs.length - 1;
		int[] xs = new int[n];
		int[] ys = new int[n];
		for (int i = 0; i < n; i++) {
			xs[i] = (int)cs[i].x;
			ys[i] = (int)cs[i].y;
		}
		return new PolygonRoi(xs, ys, n, PolygonRoi.POLYGON);
	}
	
	public void clearAll() {
		for (Integer i : rois.keySet()) {
			clear(i);
		}
	}
	
	public void clear(int frame) {
		this.rois.get(frame).clear();
		imp.updateAndDraw();
	}
	
	public void copyRois(int source, int dest) {
		if (source > 0 && dest > 0 && source <= imp.getStackSize() && dest <= imp.getStackSize()) {
			rois.get(dest).addAll(rois.get(source));
			imp.updateAndDraw();
		}
	}
	

	public Map<String,Integer> getPositions() {
		Map<String,Integer> positions = new HashMap<String,Integer>();
		String[] labels = imp.getStack().getSliceLabels();
		for (int i = 0; i < imp.getNSlices(); i++) {
			positions.put(labels[i],i);
		}
		return positions;
	}
}
