//	Copyright 2007 Fred Ross
//
//	This file is part of MultiRoi.
//
//    MultiRoi is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    MultiRoi is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.Button;
import java.awt.GridLayout;
import java.awt.Dimension;
import java.awt.event.*;
import ij.gui.*;
import ij.ImagePlus;
import java.io.*;
import ij.IJ;
import java.util.*;

public class MultiRoiToolsPanel implements ActionListener {
	private Button btnClear, btnCopyFromLast, btnCopyToLast, btnCopyFromNext, 
					btnCopyToNext, btnReadRois, btnWriteRois, btnDeleteCurrent;
	MultiRoiStackCanvas ic;
	ImagePlus imp;
	MultiRoiToolsPanel(ImagePlus imp, MultiRoiStackCanvas ic) {
		this.ic = ic;
		this.imp = imp;
		Frame panel = new Frame("MultiRoi Tools");
		GridLayout layout = new GridLayout(7,1);
		panel.setLayout(layout);
		btnClear = new Button("Clear ROIs in frame");
		btnClear.addActionListener(this);
		panel.add(btnClear);

		btnCopyFromLast = new Button("Copy ROIs from previous frame");
		btnCopyFromLast.addActionListener(this);
		panel.add(btnCopyFromLast);

		btnCopyToLast = new Button("Copy ROIs to previous frame");
		btnCopyToLast.addActionListener(this);
		panel.add(btnCopyToLast);

		btnCopyToNext = new Button("Copy ROIs to next frame");
		btnCopyToNext.addActionListener(this);
		panel.add(btnCopyToNext);

		btnCopyFromNext = new Button("Copy ROIs from next frame");
		btnCopyFromNext.addActionListener(this);
		panel.add(btnCopyFromNext);

		btnReadRois = new Button("Read ROIs from file");
		btnReadRois.addActionListener(this);
		panel.add(btnReadRois);

		btnWriteRois = new Button("Write ROIs to file");
		btnWriteRois.addActionListener(this);
		panel.add(btnWriteRois);
		
		btnDeleteCurrent = new Button("Delete current ROI");
		btnDeleteCurrent.addActionListener(this);
		panel.add(btnDeleteCurrent);
		//panel.setSize(panel.getPreferredSize());
		panel.setSize(new Dimension(200,200));
		panel.setVisible(true);
	}
	
	public void actionPerformed(ActionEvent e) {
		Object b = e.getSource();
		if (b == btnClear) {
			ic.clear(imp.getCurrentSlice());
		} else if (b == btnCopyFromLast) {
			ic.copyRois(imp.getCurrentSlice()-1,imp.getCurrentSlice());
		} else if (b == btnCopyToLast) {
			ic.copyRois(imp.getCurrentSlice(), imp.getCurrentSlice()-1);
		} else if (b == btnCopyFromNext) {
			ic.copyRois(imp.getCurrentSlice()+1, imp.getCurrentSlice());
		} else if (b == btnCopyToNext) {
			ic.copyRois(imp.getCurrentSlice(), imp.getCurrentSlice()+1);
		} else if (b == btnDeleteCurrent) {
			imp.setRoi((Roi)null);
		} else if (b == btnWriteRois) {
			writeRois();
		} else if (b == btnReadRois) {
			readRois();
		}
	}

	public void writeRois() {
		Frame frame = new Frame();
		FileDialog fd = new FileDialog(frame, "Select file to write Rois to", FileDialog.SAVE);
		fd.show();
		String file = fd.getFile();
		String directory = fd.getDirectory();
		if (file != null) {
			try {
				File f = new File(directory, file);
				PrintWriter out = new PrintWriter(new FileWriter(f));
				out.print(RoiSetParser.encodeEntries(ic.getPositions(), ic.rois));
				out.close();
			} catch (IOException e) {
				IJ.error("Couldn't write Rois to " + file + ": " + e.getMessage());
			}
		}
	}
	
	public void readRois() {
		Frame scframe = new Frame();
		SaveChangesDialog scd = new SaveChangesDialog(scframe, "Rois");
		if (scd.cancelPressed()) {
			return;
		} else if (scd.savePressed()) {
			writeRois();
		}
		Frame fdframe = new Frame();
		FileDialog fd = new FileDialog(fdframe, "Select file containing Rois", FileDialog.LOAD);
		fd.show();
		String file = fd.getFile();
		String directory = fd.getDirectory();
		if (file == null) {
			return;
		} else {
			try {
				StringBuffer sb = new StringBuffer();
				File f = new File(directory, file);
				BufferedReader in = new BufferedReader(new FileReader(f));
				String line;
				while ((line = in.readLine()) != null) {
					sb.append(line);
				}
				ic.clearAll();
				Map<Integer,List<Roi>> newrois = RoiSetParser.decodeEntries(sb.toString(), ic.getPositions());
				for (Integer ptr : newrois.keySet()) {
					ic.rois.get(ptr).addAll(newrois.get(ptr));
				}
			} catch (IOException e) {
				IJ.error("Error reading from " + file + ": " + e.getMessage());
			} catch (com.vividsolutions.jts.io.ParseException pe) {
				IJ.error("Error parsing " + file + ": " + pe.getMessage());
			}
		}
	}
}
