import ij.gui.Roi;
import ij.IJ;
import com.vividsolutions.jts.io.WKTReader;
import com.vividsolutions.jts.io.ParseException;
import java.util.*;



public class RoiSetParser {
	public static String encodeRoi(Roi r) {
		return(MultiRoiStackCanvas.roiToGeometry(r).toText());
	}
	
	public static Roi decodeRoi(String s) throws ParseException {
		try {
			WKTReader reader = new WKTReader();
			return(MultiRoiStackCanvas.geometryToRoi(reader.read(s)));
		} catch (ParseException e) {
			IJ.error("Invalid WKT string: " + e.getMessage());
			throw e;
		}
	}
	
	public static List<Roi> decodeRoiList(String s) throws ParseException {
		String[] wkts = s.split(";");
		ArrayList<Roi> rs = new ArrayList<Roi>();
		for (int i = 0; i < wkts.length; i++) {
			rs.add(decodeRoi(wkts[i].trim()));
		}
		return rs;
	}

	public static String join( String token, String[] strings )
	{
		StringBuffer sb = new StringBuffer();
		
		for( int x = 0; x < ( strings.length - 1 ); x++ )
		{
			sb.append( strings[x] );
			sb.append( token );
		}
		sb.append( strings[ strings.length - 1 ] );
		
		return( sb.toString() );
	}
	
	public static String encodeRoiList(List<Roi> rs0) {
		Roi[] rs = new Roi[rs0.size()];
		rs = rs0.toArray(rs);
		String[] encodings = new String[rs.length];
		for (int i = 0; i < rs.length; i++) {
			encodings[i] = encodeRoi(rs[i]);
		}
		return(join(";", encodings));
	}
	
	public static String encodeEntry(String label, List<Roi> rs0) {
		return(label + "{" + encodeRoiList(rs0) + "}");
	}
	
	public static class Entry {
		String label;
		List<Roi> rois;
	}
	
	public static Entry decodeEntry(String s) throws ParseException {
		Entry k = new Entry();
		String[] fields = s.split("\\{");
		k.label = fields[0].trim();
		k.rois = decodeRoiList(fields[1].trim());
		return k;
	}
	
	public static String encodeEntries(Map<String,Integer> labels, Map<Integer, List<Roi>> rs0) {
		ArrayList<String> entries = new ArrayList<String>(labels.size());
		for (String lbl : labels.keySet()) {
			Integer ptr = labels.get(lbl);
			if (rs0.containsKey(ptr) && rs0.get(ptr).size()>0) {
				entries.add(encodeEntry(lbl, rs0.get(ptr)));
			}
		}
		String[] k = new String[entries.size()];
		return(join("\n", entries.toArray(k)));
	}
	
	public static Map<Integer,List<Roi>> decodeEntries(String s, Map<String,Integer> positions) 
	throws ParseException 
	{
		Map<Integer,List<Roi>> framemap = new HashMap<Integer,List<Roi>>();
		String[] entries = s.split("}");
		for (int i = 0; i < entries.length; i++) { // length-1 because we have a null string at the end from split 
			Entry e = decodeEntry(entries[i]);
			if (positions.containsKey(e.label)) {
				Integer ptr = positions.get(e.label);
				framemap.put(ptr, e.rois);
			}
		}
		return framemap;
	}
	
}
