import org.junit.*;
import junit.framework.TestCase;
import ij.gui.*;
import com.vividsolutions.jts.io.ParseException;
import java.util.*;

public class RoiSetParserTest extends TestCase {
	final String expectedr = "POLYGON ((10 10, 20 10, 20 20, 10 20, 10 10, 10 10))";
	PolygonRoi r;
	List<Roi> rs;
	final String expectedrs = expectedr + ";" + expectedr + ";" + expectedr + ";" + expectedr;
	final String exblock = "{" + expectedrs + "}";
	final String label = "point05_w528_t034";
	final String label2 = "point05_w528_t035";
	final String label3 = "point05_w528_t033";
	final String label4 = "point05_w528_t036";
	final String expectedEntry = label + exblock;
	final String expectedEntries = label+exblock+"\n"+label2+exblock+"\n"+label3+exblock + "\n" + label4+exblock;
	final String mungedEntries = label+exblock + "\n" + label2+exblock + label3+exblock + "\n\t\t  " + label4+exblock;
	Map<Integer, List<Roi>> framemap;
	Map<String, Integer> positions; 
	Map<String, Integer> subpositions;
	final String expectedsubEntries = label+exblock+"\n"+label2+exblock+"\n"+label3+exblock;
	
	@Before
	public void setUp() throws Exception {
		int[] xs = {10, 20, 20, 10, 10};
		int[] ys = {10, 10, 20, 20, 10};
		this.r = new PolygonRoi(xs, ys, 5, PolygonRoi.POLYGON);
		rs = new ArrayList<Roi>(4);
		rs.add(this.r);
		rs.add(this.r);
		rs.add(this.r);
		rs.add(this.r);
		framemap = new HashMap<Integer, List<Roi>>();
		framemap.put(1, rs);
		framemap.put(2, rs);
		framemap.put(4, rs);
		framemap.put(7, rs);
		positions = new HashMap<String, Integer>();
		positions.put(label, 1);
		positions.put(label2, 2);
		positions.put(label3, 4);
		positions.put(label4, 7);
		subpositions = new HashMap<String,Integer>();
		subpositions.put(label,1);
		subpositions.put(label2,2);
		subpositions.put(label3,4);
	}
	
	public void testEncodeRoi() {
		String encoded = RoiSetParser.encodeRoi(this.r);
		assertEquals(this.expectedr, encoded);
	}

	public void testDecodeRoi() {
		String txt = "POLYGON ((10 10, 20 10, 20 20, 10 20, 10 10))";
		try {
			String ntxt = RoiSetParser.encodeRoi(RoiSetParser.decodeRoi(txt));
			assertEquals(txt,ntxt);
		} catch (ParseException e) {}
	}
	
	public void testEncodeRoiList() {
		String actual = RoiSetParser.encodeRoiList(this.rs);
		assertEquals(this.expectedrs, actual);
	}
	
	public void testDecodeRoiList() {
		try { 
			assertEquals(this.expectedrs, 
					RoiSetParser.encodeRoiList(RoiSetParser.decodeRoiList(this.expectedrs)));
		} catch (ParseException e) {}
	}

	public void testEncodeEntry() {
		assertEquals(expectedEntry, RoiSetParser.encodeEntry(label, rs));
	}
	
	public void testDecodeEntry() {
		try {
			RoiSetParser.Entry e = RoiSetParser.decodeEntry(expectedEntry);
			assertEquals(e.label, this.label);
			assertEquals(expectedrs, RoiSetParser.encodeRoiList(e.rois));
		} catch (ParseException e) {}
	}
	
	public void testEncodeEntries() {
		assertEquals(expectedEntries, RoiSetParser.encodeEntries(positions, framemap));
	}
	
	public void testDecodeEntries() {
		try { 
			Map<Integer,List<Roi>> k = RoiSetParser.decodeEntries(expectedEntries, positions);
			Map<Integer,List<Roi>> subk = RoiSetParser.decodeEntries(expectedEntries, subpositions);
			assertEquals(expectedEntries, RoiSetParser.encodeEntries(positions, k));
			assertEquals(expectedsubEntries, RoiSetParser.encodeEntries(positions, subk));
		} catch (ParseException e) {}
	}
}
